#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley
# Copyright (c) 2019-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#

import sys
import unittest

from pyasn1.codec.der.decoder import decode as der_decoder
from pyasn1.codec.der.encoder import encode as der_encoder

from pyasn1_alt_modules import pem
from pyasn1_alt_modules import rfc3657
from pyasn1_alt_modules import rfc5652
from pyasn1_alt_modules import rfc5751
from pyasn1_alt_modules import opentypemap


class EnvelopedDataTestCase(unittest.TestCase):
    env_data_pem_text = """\
MIIFfwYJKoZIhvcNAQcDoIIFcDCCBWwCAQIxU6JRAgEEMCMEECBlcTFnxBsPlsug
4KOCj78YDzIwMTkwOTEyMTIwMDAwWjANBgsqgwiMmks9AQEDAgQYS3mK9jQmvth1
iuBV8PEa89ICvmoomJCvMIIFEAYJKoZIhvcNAQcBMB8GCyqDCIyaSz0BAQECBBBC
T0dVU0lWX0JPR1VTSVYhgIIE4HPHsXoYyQ/4LRDiK4OrSuRJmmuDye5fH/hLcgw/
330Gsl1QBs9jF1CEDBM5ki657K/TRMl78Rqb3LIu5lfLQ8WVNGLsoQPwvxzIexGg
ShtYYwu8TcPiESFMa20SWpDEG8zFlmCbqQuc0buPxnvYviVThoBEthNC+S2Umed8
JpxwNKJbNTx5dxd2dkDNwpHsKgNzT9cGl0NF129Dspehqtdge5LJu3rj1gNynLRI
32AQ+pwU+sEHee6wDHhU5OWnHlndkm/9MTKY3woOhs1/KQFlRFPC6k71ZpUlncd3
93wLVIImfoMe4zWPAOnbpZ/M7zEJ95rTwwmudBs0qwMfCa3h0Vkg69w6fBHyc1IH
8u3VpSPcbOW4dUzJBDJPgB1kObAV02ZA4FQEuZtZiG13u3c7sSrHxsY1rtXssvSe
+5rThqPWgDqmH8b/yPGEHIFh03kHCDt/UZrdkLCO7a0WhCdY4I9hNU6OYEQmyEFs
0LsqEumn34Lv/XcD1wgLdPtF65zub4Wil/0Vpu73vIWLIk9LyNIXQSd6w0ZHUvVS
+jZZ1zrqIQKhKvG97NpKAYoHa4tOdoXHgBJUxw/uAOKkQ4jC5RS5UKqCZaQcArRD
2bCEEsutiuyf06MMcWm+RaBY1EwuX+/cT0D6CsWHYFAeQHgLuR4HVk5+PVKoOL/7
KUz0jUU5gzFVcmfaocyX5A6R90yggBObefcOIEj3v+5fjHkppfTvi/R03fVZ4Nyw
WyHbN7kOHHy8skJpcvNaqSY0dfkb8KOOoTptJH9rCBYtFlC5j/18y8Om9Um4h3/4
6hYO0xU8izJDzDzJnO/5KS5mGyskweIp3mrE1C/mw68LvrksxQI03CPtbM+FqOKe
0VcsAQykiOTnG3d4jLeF1iVrc9CgV+pwc5VfgQUwsGhjAFOCKTwWDrr3Je0yVsfz
gwY2zuM5uE/+usOSBt7SqbFTLOCba4fJrVVwi0wZig88owVTdl/ACxl2qyLUYC2u
5PNJSY6kx8Cgo4gDJk/3oeuys8JqgaufvKybl5GsdDaF3A7usZAjDR1EAWHZ7JGi
agtqbvISLD0zq4e4nmEhLnIRb7u5SNBPqe8qVuuQjIsvmP0ZuTlnh84ypFOQGz7c
fzHtr6UEQoGj8HImbp8diL4tflmFAVNaRjQzu18+2vFB2w1EZIe2/uNLs9ne2EIy
oK2Qb+mMCwJsNS0xOG0/TzPZ+y0Tp1/LupLHovMosPIGXlbvqZVh2xftDvbIigIM
WZQZ2tFxYD6Xc4zA00v7H0yGF1pRY+3GpobJkw0Y6ORtgdtdnr2ipioIeQCy0hUp
POmTeSr0L3H7KfNY7yQgZg0ra7FIEjM8tDoNqrhznetYUU1ZWM8Lyb3zMxxinSFs
GFGx2TiqPyixJNxN+lPT5D6GRhC9mXgh+BfVod5oINJJwXxJpT5xnsZgW8ujVxiu
1Vt5esXCZaXTGlyjVTH5dmCvJP9+B8n7dOimmCxCbMQKpNaZixJhoXWQtTgKqL1Q
f9WoEs6TDGgfTllqjbE4w3O7ZA7fAWe9jbAGwiPV5rF/NVvjaj2+ibtXbSNPW59d
dy1/2WzknVYnEHF0qZdBZ02Wh4ByXUC3FNvDu8hRTm5aq73DCqXLXUwNU8BvS1xB
bbRq5aYI2Rd3naNAns9dHqSvkg==
"""

    def setUp(self):
        self.asn1Spec = rfc5652.ContentInfo()

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.env_data_pem_text)
        asn1Object, rest = der_decoder(substrate, asn1Spec=self.asn1Spec)
        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))
        self.assertEqual(rfc5652.id_envelopedData, asn1Object['contentType'])

        ed, rest = der_decoder(
            asn1Object['content'], asn1Spec=rfc5652.EnvelopedData())
        self.assertFalse(rest)
        self.assertTrue(ed.prettyPrint())
        self.assertEqual(asn1Object['content'], der_encoder(ed))

        kwa = ed['recipientInfos'][0]['kekri']['keyEncryptionAlgorithm']
        self.assertEqual(rfc3657.id_camellia128_wrap, kwa['algorithm'])

        cea = ed['encryptedContentInfo']['contentEncryptionAlgorithm']
        self.assertEqual(rfc3657.id_camellia128_cbc, cea['algorithm'])
        param, rest = der_decoder(
            cea['parameters'], asn1Spec=rfc3657.Camellia_IV())
        self.assertFalse(rest)
        self.assertTrue(param.prettyPrint())
        self.assertEqual(cea['parameters'], der_encoder(param))

        iv = rfc3657.Camellia_IV(hexValue='424f47555349565f424f475553495621')
        self.assertEqual(iv, param)

    def testOpenTypes(self):
        substrate = pem.readBase64fromText(self.env_data_pem_text)
        asn1Object, rest = der_decoder(
            substrate, asn1Spec=self.asn1Spec, decodeOpenTypes=True)
        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))
        cmsContentTypesMap = opentypemap.get('cmsContentTypesMap')
        self.assertIn(asn1Object['contentType'], cmsContentTypesMap)

        kekri = asn1Object['content']['recipientInfos'][0]['kekri']
        kwa = kekri['keyEncryptionAlgorithm']
        self.assertEqual(rfc3657.id_camellia128_wrap, kwa['algorithm'])

        eci = asn1Object['content']['encryptedContentInfo']
        cea = eci['contentEncryptionAlgorithm']
        self.assertEqual(rfc3657.id_camellia128_cbc, cea['algorithm'])

        iv = rfc3657.Camellia_IV(hexValue='424f47555349565f424f475553495621')
        self.assertEqual(iv, cea['parameters'])

class SMIMECapabilitiesTestCase(unittest.TestCase):
    smime_capabilities_pem_text = """\
MGYwDwYLKoMIjJpLPQEBAQIFADAPBgsqgwiMmks9AQEBAwUAMA8GCyqDCIyaSz0B
AQEEBQAwDwYLKoMIjJpLPQEBAwIFADAPBgsqgwiMmks9AQEDAwUAMA8GCyqDCIya
Sz0BAQMEBQA=
"""

    def setUp(self):
        self.asn1Spec = rfc5751.SMIMECapabilities()

    def testDerCodec(self):
        alg_oid_list = [
            rfc3657.id_camellia128_cbc,
            rfc3657.id_camellia192_cbc,
            rfc3657.id_camellia256_cbc,
            rfc3657.id_camellia128_wrap,
            rfc3657.id_camellia192_wrap,
            rfc3657.id_camellia256_wrap,
        ]

        substrate = pem.readBase64fromText(self.smime_capabilities_pem_text)
        asn1Object, rest = der_decoder(substrate, asn1Spec=self.asn1Spec)
        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))
 
        param = der_encoder(rfc3657.CamelliaSMimeCapability(""))
        count = 0
        for cap in asn1Object:
            self.assertEqual(cap['parameters'], param)
            self.assertIn(cap['capabilityID'], alg_oid_list)
            count += 1

        self.assertEqual(count, 6)

    def testOpenTypes(self):
        substrate = pem.readBase64fromText(self.smime_capabilities_pem_text)
        asn1Object, rest = der_decoder(
            substrate, asn1Spec=self.asn1Spec, decodeOpenTypes=True)
        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))
 
        smimeCapabilityMap = opentypemap.get('smimeCapabilityMap')
        param = rfc3657.CamelliaSMimeCapability("")
        count = 0
        for cap in asn1Object:
            self.assertIn(cap['capabilityID'], smimeCapabilityMap)
            self.assertEqual(cap['parameters'], param)
            count += 1

        self.assertEqual(count, 6)


suite = unittest.TestLoader().loadTestsFromModule(sys.modules[__name__])

if __name__ == '__main__':
    result = unittest.TextTestRunner(verbosity=2).run(suite)
    sys.exit(not result.wasSuccessful())



# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley
# Modified by Russ Housley to import items related to KDF2 and KDF3 from
#   rfc5990, which makes the algorithmIdentifierMap update more simple.
#
# Copyright (c) 2024-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# Use of the SHA3 One-way Hash Functions in the CMS
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc9688.txt

from pyasn1.type import univ

from pyasn1_alt_modules import rfc3279
from pyasn1_alt_modules import rfc5280
from pyasn1_alt_modules import rfc5990

from pyasn1_alt_modules import opentypemap

algorithmIdentifierMap = opentypemap.get('algorithmIdentifierMap')

smimeCapabilityMap = opentypemap.get('smimeCapabilityMap')


# Imports from RFC 3279

rsaEncryption = rfc3279.rsaEncryption

RSAPublicKey = rfc3279.RSAPublicKey

ECPoint = rfc3279.ECPoint

ECDSA_Sig_Value = rfc3279.ECDSA_Sig_Value


# Imports from RFC 5280

AlgorithmIdentifier = rfc5280.AlgorithmIdentifier


# Imports from RFC 5990

x9_44 = rfc5990.x9_44

x9_44_components = rfc5990.x9_44_components

id_kdf_kdf2 = rfc5990.id_kdf_kdf2

id_kdf_kdf3 = rfc5990.id_kdf_kdf3

KDF2_HashFunction = rfc5990.KDF2_HashFunction

KDF3_HashFunction = rfc5990.KDF3_HashFunction

KeyDerivationFunction = rfc5990.KeyDerivationFunction


# OID  arcs

nistAlgorithm = univ.ObjectIdentifier('2.16.840.1.101.3.4')

hashAlgs = nistAlgorithm + (2, )

sigAlgs = nistAlgorithm + (3, )

id_alg = univ.ObjectIdentifier('1.2.840.113549.1.9.16.3')


# SHA3 Hash Algorithms

id_sha3_224 = hashAlgs + (7, )

id_sha3_256 = hashAlgs + (8, )

id_sha3_384 = hashAlgs + (9, )

id_sha3_512 = hashAlgs + (10, )

mda_sha3_224 = rfc5280.AlgorithmIdentifier()
mda_sha3_224['algorithm'] = id_sha3_224
# mda_id_sha3_224['parameters'] is absent

mda_sha3_256 = rfc5280.AlgorithmIdentifier()
mda_sha3_256['algorithm'] = id_sha3_256
# mda_id_sha3_256['parameters'] is absent

mda_sha3_384 = rfc5280.AlgorithmIdentifier()
mda_sha3_384['algorithm'] = id_sha3_384
# mda_id_sha3_384['parameters'] is absent

mda_sha3_512 = rfc5280.AlgorithmIdentifier()
mda_sha3_512['algorithm'] = id_sha3_512
# mda_id_sha3_512['parameters'] is absent

class HashAlgorithm(AlgorithmIdentifier):
    pass


# RSASSA PKCS#1 v1.5 with SHA3

id_rsassa_pkcs1_v1_5_with_sha3_224 = sigAlgs + (13, )

id_rsassa_pkcs1_v1_5_with_sha3_256 = sigAlgs + (14, )

id_rsassa_pkcs1_v1_5_with_sha3_384 = sigAlgs + (15, )

id_rsassa_pkcs1_v1_5_with_sha3_512 = sigAlgs + (16, )
      
sa_rsassa_pkcs1_v1_5_with_sha3_224 = rfc5280.AlgorithmIdentifier()
sa_rsassa_pkcs1_v1_5_with_sha3_224['algorithm'] = id_rsassa_pkcs1_v1_5_with_sha3_224
sa_rsassa_pkcs1_v1_5_with_sha3_224['parameters'] = univ.Null('')

sa_rsassa_pkcs1_v1_5_with_sha3_256 = rfc5280.AlgorithmIdentifier()
sa_rsassa_pkcs1_v1_5_with_sha3_256['algorithm'] = id_rsassa_pkcs1_v1_5_with_sha3_256
sa_rsassa_pkcs1_v1_5_with_sha3_256['parameters'] = univ.Null('')

sa_rsassa_pkcs1_v1_5_with_sha3_384 = rfc5280.AlgorithmIdentifier()
sa_rsassa_pkcs1_v1_5_with_sha3_384['algorithm'] = id_rsassa_pkcs1_v1_5_with_sha3_384
sa_rsassa_pkcs1_v1_5_with_sha3_384['parameters'] = univ.Null('')

sa_rsassa_pkcs1_v1_5_with_sha3_512 = rfc5280.AlgorithmIdentifier()
sa_rsassa_pkcs1_v1_5_with_sha3_512['algorithm'] = id_rsassa_pkcs1_v1_5_with_sha3_512
sa_rsassa_pkcs1_v1_5_with_sha3_512['parameters'] = univ.Null('')

pk_rsassa_pkcs1_v1_5_with_sha3_224 = rfc5280.SubjectPublicKeyInfo()
pk_rsassa_pkcs1_v1_5_with_sha3_224['algorithm'] = sa_rsassa_pkcs1_v1_5_with_sha3_224
# pk_rsassa_pkcs1_v1_5_with_sha3_224['subjectPublicKey'] is DER-encoded RSAPublicKey

pk_rsassa_pkcs1_v1_5_with_sha3_256 = rfc5280.SubjectPublicKeyInfo()
pk_rsassa_pkcs1_v1_5_with_sha3_256['algorithm'] = sa_rsassa_pkcs1_v1_5_with_sha3_256
# pk_rsassa_pkcs1_v1_5_with_sha3_256['subjectPublicKey'] is DER-encoded RSAPublicKey

pk_rsassa_pkcs1_v1_5_with_sha3_384 = rfc5280.SubjectPublicKeyInfo()
pk_rsassa_pkcs1_v1_5_with_sha3_384['algorithm'] = sa_rsassa_pkcs1_v1_5_with_sha3_384
# pk_rsassa_pkcs1_v1_5_with_sha3_384['subjectPublicKey'] is DER-encoded RSAPublicKey

pk_rsassa_pkcs1_v1_5_with_sha3_512 = rfc5280.SubjectPublicKeyInfo()
pk_rsassa_pkcs1_v1_5_with_sha3_512['algorithm'] = sa_rsassa_pkcs1_v1_5_with_sha3_512
# pk_rsassa_pkcs1_v1_5_with_sha3_512['subjectPublicKey'] is DER-encoded RSAPublicKey


# ECDSA with SHA3

id_ecdsa_with_sha3_224 = sigAlgs + (9, )

id_ecdsa_with_sha3_256 = sigAlgs + (10, )

id_ecdsa_with_sha3_384 = sigAlgs + (11, )

id_ecdsa_with_sha3_512 = sigAlgs + (12, )

sa_ecdsa_with_sha3_224 = rfc5280.AlgorithmIdentifier()
sa_ecdsa_with_sha3_224['algorithm'] = id_ecdsa_with_sha3_224
# sa_ecdsa_with_sha3_224['parameters'] is absent

sa_ecdsa_with_sha3_256 = rfc5280.AlgorithmIdentifier()
sa_ecdsa_with_sha3_256['algorithm'] = id_ecdsa_with_sha3_256
# sa_ecdsa_with_sha3_256['parameters'] is absent

sa_ecdsa_with_sha3_384 = rfc5280.AlgorithmIdentifier()
sa_ecdsa_with_sha3_384['algorithm'] = id_ecdsa_with_sha3_384
# sa_ecdsa_with_sha3_384['parameters'] is absent

sa_ecdsa_with_sha3_512 = rfc5280.AlgorithmIdentifier()
sa_ecdsa_with_sha3_512['algorithm'] = id_ecdsa_with_sha3_512
# sa_ecdsa_with_sha3_512['parameters'] is absent

pk_ecdsa_with_sha3_224 = rfc5280.SubjectPublicKeyInfo()
pk_ecdsa_with_sha3_224['algorithm'] = sa_ecdsa_with_sha3_224
# pk_ecdsa_with_sha3_224['subjectPublicKey'] is DER-encoded ECPoint

pk_ecdsa_with_sha3_256 = rfc5280.SubjectPublicKeyInfo()
pk_ecdsa_with_sha3_256['algorithm'] = sa_ecdsa_with_sha3_256
# pk_ecdsa_with_sha3_256['subjectPublicKey'] is DER-encoded ECPoint

pk_ecdsa_with_sha3_384 = rfc5280.SubjectPublicKeyInfo()
pk_ecdsa_with_sha3_384['algorithm'] = sa_ecdsa_with_sha3_384
# pk_ecdsa_with_sha3_384['subjectPublicKey'] is DER-encoded ECPoint

pk_ecdsa_with_sha3_512 = rfc5280.SubjectPublicKeyInfo()
pk_ecdsa_with_sha3_512['algorithm'] = sa_ecdsa_with_sha3_512
# pk_ecdsa_with_sha3_512['subjectPublicKey'] is DER-encoded ECPoint

class SignatureAlgorithm(AlgorithmIdentifier):
    pass


# HMAC with SHA3

id_hmacWithSHA3_224 = hashAlgs + (13, )

id_hmacWithSHA3_256 = hashAlgs + (14, )

id_hmacWithSHA3_384 = hashAlgs + (15, )

id_hmacWithSHA3_512 = hashAlgs + (16, )

maca_hmacWithSHA3_224 = rfc5280.AlgorithmIdentifier()
maca_hmacWithSHA3_224['algorithm'] = id_hmacWithSHA3_224
# maca_hmacWithSHA3_224['parameters'] are absent

maca_hmacWithSHA3_256 = rfc5280.AlgorithmIdentifier()
maca_hmacWithSHA3_256['algorithm'] = id_hmacWithSHA3_256
# maca_hmacWithSHA3_256['parameters'] are absent

maca_hmacWithSHA3_384 = rfc5280.AlgorithmIdentifier()
maca_hmacWithSHA3_384['algorithm'] = id_hmacWithSHA3_384
# maca_hmacWithSHA3_384['parameters'] are absent

maca_hmacWithSHA3_512 = rfc5280.AlgorithmIdentifier()
maca_hmacWithSHA3_512['algorithm'] = id_hmacWithSHA3_512
# maca_hmacWithSHA3_512['parameters'] are absent

class MACAlgorithm(AlgorithmIdentifier):
    pass


# HKDF with SHA3

id_alg_hkdf_with_sha3_224 = id_alg + (32, )

id_alg_hkdf_with_sha3_256 = id_alg + (33, )

id_alg_hkdf_with_sha3_384 = id_alg + (34, )

id_alg_hkdf_with_sha3_512 = id_alg + (35, )

kda_hkdf_with_sha3_224 = rfc5280.AlgorithmIdentifier()
kda_hkdf_with_sha3_224['algorithm'] = id_alg_hkdf_with_sha3_224
# kda_hkdf_with_sha3_224['parameters'] are absent

kda_hkdf_with_sha3_256 = rfc5280.AlgorithmIdentifier()
kda_hkdf_with_sha3_256['algorithm'] = id_alg_hkdf_with_sha3_256
# kda_hkdf_with_sha3_256['parameters'] are absent

kda_hkdf_with_sha3_384 = rfc5280.AlgorithmIdentifier()
kda_hkdf_with_sha3_384['algorithm'] = id_alg_hkdf_with_sha3_384
# kda_hkdf_with_sha3_384['parameters'] are absent

kda_hkdf_with_sha3_512 = rfc5280.AlgorithmIdentifier()
kda_hkdf_with_sha3_512['algorithm'] = id_alg_hkdf_with_sha3_512
# kda_hkdf_with_sha3_512['parameters'] are absent


# KDF using KMAC128 and KMAC512

id_kmac128 = hashAlgs + (21, )

id_kmac256 = hashAlgs + (22, )

class Customization(univ.OctetString):
    pass

kda_kmac128 = rfc5280.AlgorithmIdentifier()
kda_kmac128['algorithm'] = id_kmac128
# kda_kmac128['parameters'] are absent when Customization is ''H

kda_kmac256 = rfc5280.AlgorithmIdentifier()
kda_kmac256['algorithm'] = id_kmac256
# kda_kmac256['parameters'] are absent when Customization is ''H


# KDF2 and KDF3 with SHA3

kda_kdf2 = rfc5280.AlgorithmIdentifier()
kda_kdf2['algorithm'] = id_kdf_kdf2
kda_kdf2['parameters'] = id_sha3_256
# kda_kdf2['parameters'] can be the OID for any hash function

kda_kdf3 = rfc5280.AlgorithmIdentifier()
kda_kdf3['algorithm'] = id_kdf_kdf3
kda_kdf3['parameters'] = id_sha3_256
# kda_kdf3['parameters'] can be the OID for any hash function


# Update the algorithm identifiers map and the S/MIME capability map
#
# No need to add all of the OIDs to the algorithmIdentifierMap and the
# smimeCapabilityMap; do not add the ones where the parameters are
# always absent.  Also, the KDF OIDs do not get added to the S/MIME
# capability map.

_mapUpdate = {
    id_rsassa_pkcs1_v1_5_with_sha3_224: univ.Null(),
    id_rsassa_pkcs1_v1_5_with_sha3_256: univ.Null(),
    id_rsassa_pkcs1_v1_5_with_sha3_384: univ.Null(),
    id_rsassa_pkcs1_v1_5_with_sha3_512: univ.Null(),
    id_kmac128: Customization(),
    id_kmac256: Customization(),
}

algorithmIdentifierMap.update(_mapUpdate)

smimeCapabilityMap.update(_mapUpdate)



#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
#
# Copyright (c) 2019-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# Diffie-Hellman Key Agreement
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc2631.txt
# https://www.rfc-editor.org/errata/eid5897
#

from pyasn1.type import constraint
from pyasn1.type import namedtype
from pyasn1.type import tag
from pyasn1.type import univ


class KeySpecificInfo(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('algorithm', univ.ObjectIdentifier()),
        namedtype.NamedType('counter', univ.OctetString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(4, 4)))
    )


class OtherInfo(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('keyInfo', KeySpecificInfo()),
        namedtype.OptionalNamedType('partyAInfo', univ.OctetString().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0))),
        namedtype.NamedType('suppPubInfo', univ.OctetString().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)))
    )



# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley.
# Modified by Russ Housley to import SCVP-related structures from RFC 5055,
#   which did not exist at the time this module was first written.  Also,
#   include the opentypemap manager.
#
# Copyright (c) 2019-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# Other Certificates Extension
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc5697.txt

from pyasn1.type import namedtype
from pyasn1.type import univ

from pyasn1_alt_modules import rfc5280
from pyasn1_alt_modules import rfc4055
from pyasn1_alt_modules import rfc5055
from pyasn1_alt_modules import opentypemap

certificateExtensionsMap = opentypemap.get('certificateExtensionsMap')


# Imports from RFC 5280

AlgorithmIdentifier = rfc5280.AlgorithmIdentifier

CertificateSerialNumber = rfc5280.CertificateSerialNumber

GeneralNames = rfc5280.GeneralNames


# Imports from RFC 4055

id_sha1 = rfc4055.id_sha1


# Imports from RFC 5055

SCVPIssuerSerial = rfc5055.SCVPIssuerSerial

sha1_alg_id = rfc5055.algid_SHA1

SCVPCertID = rfc5055.SCVPCertID


# Other Certificates Extension

id_pe_otherCerts = univ.ObjectIdentifier((1, 3, 6, 1, 5, 5, 7, 1, 19,))

class OtherCertificates(univ.SequenceOf):
    componentType = SCVPCertID()


# Update the Certificate Extension Map

_certificateExtensionsMapUpdate = {
    id_pe_otherCerts: OtherCertificates(),
}

certificateExtensionsMap.update(_certificateExtensionsMapUpdate)



#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley.
#
# Copyright (c) 2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# ML-DSA Algorithm for use in X.509 Certificats
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc9881.txt
#

from pyasn1.type import constraint
from pyasn1.type import namedtype
from pyasn1.type import  tag
from pyasn1.type import univ

from pyasn1_alt_modules import rfc5280


# Imports from RFC 5280

AlgorithmIdentifier = rfc5280.AlgorithmIdentifier

SubjectPublicKeyInfo = rfc5280.SubjectPublicKeyInfo


# Object Identifiers

nistAlgorithms = univ.ObjectIdentifier('2.16.840.1.101.3.4')

sigAlgs = nistAlgorithms + (3, )

id_ml_dsa_44 = sigAlgs + (17, )

id_ml_dsa_65 = sigAlgs + (18, )

id_ml_dsa_87 = sigAlgs + (19, )


# ML-DSA Signature Algorithms

sa_ml_dsa_44 = AlgorithmIdentifier()
sa_ml_dsa_44['algorithm'] = id_ml_dsa_44
# sa_ml_dsa_44['parameters'] are absent

sa_ml_dsa_65 = AlgorithmIdentifier()
sa_ml_dsa_65['algorithm'] = id_ml_dsa_65
# sa_ml_dsa_65['parameters'] are absent

sa_ml_dsa_87 = AlgorithmIdentifier()
sa_ml_dsa_87['algorithm'] = id_ml_dsa_87
# sa_ml_dsa_87['parameters'] are absent


# Public Key for the ML-DSA algorithm 

pk_ml_dsa_44 = SubjectPublicKeyInfo()
pk_ml_dsa_44['algorithm']['algorithm'] = id_ml_dsa_44
# pk_ml_dsa_44['algorithm']['parameters'] are absent
# pk_ml_dsa_44['subjectPublicKey'] = univ.BitString.fromOctetString(public_key)

pk_ml_dsa_65 = SubjectPublicKeyInfo()
pk_ml_dsa_65['algorithm']['algorithm'] = id_ml_dsa_65
# pk_ml_dsa_65['algorithm']['parameters'] are absent
# pk_ml_dsa_65['subjectPublicKey'] = univ.BitString.fromOctetString(public_key)

pk_ml_dsa_87 = SubjectPublicKeyInfo()
pk_ml_dsa_87['algorithm']['algorithm'] = id_ml_dsa_87
# pk_ml_dsa_87['algorithm']['parameters'] are absent
# pk_ml_dsa_87['subjectPublicKey'] = univ.BitString.fromOctetString(public_key)


# Private key for the ML-DSA algorithm outside an asymmetric key package

class ML_DSA_44_PrivateKey(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('seed', univ.OctetString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(32, 32)).subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0))),
        namedtype.NamedType('expandedKey', univ.OctetString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(2560, 2560))),
        namedtype.NamedType('both', univ.Sequence(componentType=namedtype.NamedTypes(
            namedtype.NamedType('seed', univ.OctetString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(32, 32))),
            namedtype.NamedType('expandedKey', univ.OctetString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(2560, 2560)))
        )))
    )


class ML_DSA_65_PrivateKey(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('seed', univ.OctetString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(32, 32)).subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0))),
        namedtype.NamedType('expandedKey', univ.OctetString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(4032, 4032))),
        namedtype.NamedType('both', univ.Sequence(componentType=namedtype.NamedTypes(
            namedtype.NamedType('seed', univ.OctetString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(32, 32))),
            namedtype.NamedType('expandedKey', univ.OctetString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(4032, 4032)))
        )))
    )


class ML_DSA_87_PrivateKey(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('seed', univ.OctetString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(32, 32)).subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0))),
        namedtype.NamedType('expandedKey', univ.OctetString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(4896, 4896))),
        namedtype.NamedType('both', univ.Sequence(componentType=namedtype.NamedTypes(
            namedtype.NamedType('seed', univ.OctetString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(32, 32))),
            namedtype.NamedType('expandedKey', univ.OctetString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(4896, 4896)))
        )))
    )


# Public key for the ML-DSA algorithm outside of a certificate

class ML_DSA_44_PublicKey(univ.OctetString):
    subtypeSpec = constraint.ValueSizeConstraint(1312, 1312)


class ML_DSA_65_PublicKey(univ.OctetString):
    subtypeSpec = constraint.ValueSizeConstraint(1952, 1952)


class ML_DSA_87_PublicKey(univ.OctetString):
    subtypeSpec = constraint.ValueSizeConstraint(2602, 2602)


# No need to update the Algorithm Identifier map or the S/MIME
# Capabilities map because the parameters are always absent.



#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley.
# Modified by Russ Housley to include the opentypemap manager and
#   update the S/MIME Capability Map.
#
# Copyright (c) 2019-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# The BLAKE2 Cryptographic Hash and MAC
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc7693.txt
#

from pyasn1.type import univ

from pyasn1_alt_modules import opentypemap

algorithmIdentifierMap = opentypemap.get('algorithmIdentifierMap')

smimeCapabilityMap = opentypemap.get('smimeCapabilityMap')


# BLAKE2 Object Identifiers

hashAlgs = univ.ObjectIdentifier('1.3.6.1.4.1.1722.12.2')

blake2b = hashAlgs + (1, )

id_blake2b160 = blake2b + (5, )

id_blake2b256 = blake2b + (8, )

id_blake2b384 = blake2b + (12, )

id_blake2b512 = blake2b + (16, )

blake2s = hashAlgs + (2, )

id_blake2s128 = blake2s + (4, )

id_blake2s160 = blake2s + (5, )

id_blake2s224 = blake2s + (7, )

id_blake2s256 = blake2s + (8, )


# Update the Algorithm Identifier Map and the S/MIME Capability Map

_algorithmIdentifierMapUpdate = {
    id_blake2b160: univ.Null(),
    id_blake2b256: univ.Null(),
    id_blake2b384: univ.Null(),
    id_blake2b512: univ.Null(),
    id_blake2s128: univ.Null(),
    id_blake2s160: univ.Null(),
    id_blake2s224: univ.Null(),
    id_blake2s256: univ.Null(),
}

algorithmIdentifierMap.update(_algorithmIdentifierMapUpdate)

smimeCapabilityMap.update(_algorithmIdentifierMapUpdate)



#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley
# Copyright (c) 2024-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
import sys
import unittest

from pyasn1.codec.der.decoder import decode as der_decoder
from pyasn1.codec.der.encoder import encode as der_encoder

from pyasn1_alt_modules import pem
from pyasn1_alt_modules import rfc5652
from pyasn1_alt_modules import rfc9691


class RPKISignedTALTestCase(unittest.TestCase):
    pem_text = """\
MIIKfwYJKoZIhvcNAQcCoIIKcDCCCmwCAQMxDTALBglghkgBZQMEAgEwggSJBgsq
hkiG9w0BCRABMqCCBHgEggR0MIIEcDCCAXQwDAwKTXkgbmljZSBUQTA+FhpodHRw
czovL2V4YW1wbGUuY29tL3RhLmNlchYgcnN5bmM6Ly9leGFtcGxlLmNvbS9yc3lu
Yy90YS5jZXIwggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQCxQQ12NJzG
ISfJdy58LvFEtIzepr8ayyAoSEkbXsvF6NwMRdzybrkBc7emFomJtH3hTuKqQJno
rda1ffUI1ix4gCNvTJwuCjofkXTjbDqgjV5yLb4m34z0Mw5ZWwXqQ47MgThvUT97
m/prF4eVtTBPjM9MFBnhWjDYxwjauNIWj9tjqtUJWEP86r4o3nHbACGQnWT1zbO8
7cZWmmRxQJmgxdzxr5BEst5X1KIKbITxySVVpPUsIVALTObzJr3jQGL7BlzIR6vK
q0Chu+odFoctwz+kRguKTF3mlRv3zlYTrcBm9yV7PEwHBmXtN1l67yAAoZkpB0Q1
48I/LFPGnqsrAgMBAAGgggF4MIIBdDAMDApNeSBuaWNlIFRBMD4WGmh0dHBzOi8v
ZXhhbXBsZS5jb20vdGEuY2VyFiByc3luYzovL2V4YW1wbGUuY29tL3JzeW5jL3Rh
LmNlcjCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBALFBDXY0nMYhJ8l3
Lnwu8US0jN6mvxrLIChISRtey8Xo3AxF3PJuuQFzt6YWiYm0feFO4qpAmeit1rV9
9QjWLHiAI29MnC4KOh+RdONsOqCNXnItvibfjPQzDllbBepDjsyBOG9RP3ub+msX
h5W1ME+Mz0wUGeFaMNjHCNq40haP22Oq1QlYQ/zqvijecdsAIZCdZPXNs7ztxlaa
ZHFAmaDF3PGvkESy3lfUogpshPHJJVWk9SwhUAtM5vMmveNAYvsGXMhHq8qrQKG7
6h0Why3DP6RGC4pMXeaVG/fOVhOtwGb3JXs8TAcGZe03WXrvIAChmSkHRDXjwj8s
U8aeqysCAwEAAaGCAXgwggF0MAwMCk15IG5pY2UgVEEwPhYaaHR0cHM6Ly9leGFt
cGxlLmNvbS90YS5jZXIWIHJzeW5jOi8vZXhhbXBsZS5jb20vcnN5bmMvdGEuY2Vy
MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAsUENdjScxiEnyXcufC7x
RLSM3qa/GssgKEhJG17LxejcDEXc8m65AXO3phaJibR94U7iqkCZ6K3WtX31CNYs
eIAjb0ycLgo6H5F042w6oI1eci2+Jt+M9DMOWVsF6kOOzIE4b1E/e5v6axeHlbUw
T4zPTBQZ4Vow2McI2rjSFo/bY6rVCVhD/Oq+KN5x2wAhkJ1k9c2zvO3GVppkcUCZ
oMXc8a+QRLLeV9SiCmyE8cklVaT1LCFQC0zm8ya940Bi+wZcyEeryqtAobvqHRaH
LcM/pEYLikxd5pUb985WE63AZvclezxMBwZl7TdZeu8gAKGZKQdENePCPyxTxp6r
KwIDAQABoIIEOTCCBDUwggMdoAMCAQICAQEwDQYJKoZIhvcNAQELBQAwMzExMC8G
A1UEAxMoMEVGOEU5MjZDQkE4RDYwNDEyMkUwQjlDNjMzRUJCNTE3QkE0RkYyMTAe
Fw0yMjEwMTMxMTM3NTdaFw0yMjEwMTQxMTM3NTdaMDMxMTAvBgNVBAMTKDQ4MDQy
OEI3N0EwNzVERTMwMDRFRjBFNTdDNzg1RUFBN0E3NkQ0OEQwggEiMA0GCSqGSIb3
DQEBAQUAA4IBDwAwggEKAoIBAQCpoUuZBf5mI+ZkI6KFJlX6Rt0u2GBp+WMNaY0W
cDBKm4ngiAVZdNXR5yYse9wqXdBqBVe3l7cy4MNx91bXCng7afsA18S6ar5vFlVe
42bq0LCVxdBgZrvdxgtyiQntx1IlUNZnxr8y2ZJe8K9/FwU+9zG1fn3DjrITPy6y
+o/ejQ2RI/+V9G5fbxHAccTk9+xxuwaM14j84adWajSaHmjlQPSrpOdmxIAyjQae
txmyW0HjYqm2lSeqCVE0CMQVxf59VGYq9uQWZn9yQdgJHJ5mXtoVTGIyQ5b/nvHr
IDS3raebRJQprs8b580YOdX4WqLCP4NTepYjZf/w/O1Czr6NAgMBAAGjggFSMIIB
TjAdBgNVHQ4EFgQUSAQot3oHXeMATvDlfHheqnp21I0wHwYDVR0jBBgwFoAUDvjp
Jsuo1gQSLgucYz67UXuk/yEwDgYDVR0PAQH/BAQDAgeAMC4GA1UdHwQnMCUwI6Ah
oB+GHXJzeW5jOi8vZXhhbXBsZS5jb20vdGEvdGEuY3JsMDwGCCsGAQUFBwEBBDAw
LjAsBggrBgEFBQcwAoYgcnN5bmM6Ly9leGFtcGxlLmNvbS9yc3luYy90YS5jZXIw
OgYIKwYBBQUHAQsELjAsMCoGCCsGAQUFBzALhh5yc3luYzovL2V4YW1wbGUuY29t
L3RhL3Rhay50YWswGAYDVR0gAQH/BA4wDDAKBggrBgEFBQcOAjAhBggrBgEFBQcB
BwEB/wQSMBAwBgQCAAEFADAGBAIAAgUAMBUGCCsGAQUFBwEIAQH/BAYwBKACBQAw
DQYJKoZIhvcNAQELBQADggEBAJroqz79+WXhM24E4szybvKtnT5wEJlnnsC8X9bQ
jGUSUdUoZzpqx85IrONhBTXvoXXY5iEeYj/F3xcKZffKIssxBpgG2s34SonwV/M0
NEm4vQx/xMBreU7Hkay41RHSKu8nJLBijZc4cR0y7ojwIPy2ZV1gaY8RzJGJtARv
gecYBG5ULsiEp9tE0ayYgG/vJ/+NclfHHsh6cmqSuGqFYFQI+zdlcKiA/ks900Uz
DldRoQSHdkzlndePrs0it2LLUdVoNQ9dK2rmBfyRjmErsHBbd6GiGCq3NkpTWu2H
45Sm5TVI+xCMCw7/V6NdI6J9nsGF5qeeEVhNLm7iiixeW+kxggGMMIIBiAIBA4AU
SAQot3oHXeMATvDlfHheqnp21I0wCwYJYIZIAWUDBAIBoE0wGgYJKoZIhvcNAQkD
MQ0GCyqGSIb3DQEJEAEyMC8GCSqGSIb3DQEJBDEiBCCFIgXoWqrrgxVPyEacaAzG
Tw/hUHU1MU3CG3uQpXbPgTANBgkqhkiG9w0BAQEFAASCAQCkmO0wJ8i9bX/gCuUH
ihv+iHfx54eWMo0UQHuc8HlFBCUOfsySzvrpDtV/Au806xtOIyMR5tMROVgHiDvQ
xITEnxkdwum2nnryCIcDkr3IaDd4+oEAENHhrtvAMH6yLNSljGc0UZT62EwB7vSo
FQmIGBUqFGWfaCzoYjV/ZU8l7NU2lNVY7OK5vPzXPIi80lE60J+s7eje0LjyPclk
GOXJsEVnh+vMJLuL8gHM061yrffEV+fQsjqaRtnFsYDz4tSoTFTtedo5Mm+I167Q
47JAinnmLDcJZ94nJ29Isq3rleD0/15jEsnMEcfPXsWMABrPSxhDBoQuwzNQgZaD
Eo9c
"""

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.pem_text)
        
        layers = { }
        layers.update(rfc5652.cmsContentTypesMap)
        self.assertIn(rfc9691.id_ct_signedTAL, layers)

        getNextLayer = {
            rfc5652.id_ct_contentInfo: lambda x: x['contentType'],
            rfc5652.id_signedData: lambda x: x['encapContentInfo']['eContentType'],
        }

        getNextSubstrate = {
            rfc5652.id_ct_contentInfo: lambda x: x['content'],
            rfc5652.id_signedData: lambda x: x['encapContentInfo']['eContent'],
        }

        layer = rfc5652.id_ct_contentInfo
        while layer in getNextLayer:
            asn1Object, rest = der_decoder(substrate, asn1Spec=layers[layer])
            self.assertFalse(rest)
            self.assertTrue(asn1Object.prettyPrint())
            self.assertEqual(substrate, der_encoder(asn1Object))

            substrate = getNextSubstrate[layer](asn1Object)
            layer = getNextLayer[layer](asn1Object)

        self.assertEqual(rfc9691.id_ct_signedTAL, layer)

        asn1Object, rest = der_decoder(substrate,
            asn1Spec=rfc9691.TAK())
        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))

        self.assertEqual(0, asn1Object['version'])
        self.assertIn(u'nice', asn1Object['current']['comments'][0])

        count = 0
        for url in asn1Object['current']['certificateURIs']:
            self.assertIn('example.com', url)
            count += 1
        for url in asn1Object['predecessor']['certificateURIs']:
            self.assertIn('example.com', url)
            count += 1
        for url in asn1Object['successor']['certificateURIs']:
            self.assertIn('example.com', url)
            count += 1
        self.assertEqual(6, count)

    def testOpenTypes(self):
        substrate = pem.readBase64fromText(self.pem_text)

        asn1Object, rest = der_decoder(substrate,
            asn1Spec=rfc5652.ContentInfo(), decodeOpenTypes=True)
        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))

        substrate = asn1Object['content']['encapContentInfo']['eContent']
        asn1Object, rest = der_decoder(substrate,
            asn1Spec=rfc9691.TAK(), decodeOpenTypes=True)
        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))

        self.assertEqual(0, asn1Object['version'])
        self.assertIn(u'nice', asn1Object['current']['comments'][0])

        count = 0
        for url in asn1Object['current']['certificateURIs']:
            self.assertIn('example.com', url)
            count += 1
        for url in asn1Object['predecessor']['certificateURIs']:
            self.assertIn('example.com', url)
            count += 1
        for url in asn1Object['successor']['certificateURIs']:
            self.assertIn('example.com', url)
            count += 1
        self.assertEqual(6, count)


suite = unittest.TestLoader().loadTestsFromModule(sys.modules[__name__])

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(suite)



#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
# Modified by Russ Housley to include the opentypemap manager.
#
# Copyright (c) 2021-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# TimeStampedData
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc5544.txt
#

from pyasn1.type import char
from pyasn1.type import constraint
from pyasn1.type import namedtype
from pyasn1.type import namedval
from pyasn1.type import opentype
from pyasn1.type import tag
from pyasn1.type import univ

from pyasn1_alt_modules import rfc3161
from pyasn1_alt_modules import rfc4998
from pyasn1_alt_modules import rfc5280
from pyasn1_alt_modules import rfc5652
from pyasn1_alt_modules import opentypemap

otherEvidenceMap = opentypemap.get('otherEvidenceMap')

cmsContentTypesMap = opentypemap.get('cmsContentTypesMap')

MAX = float('inf')


# Imports from RFC 5652

Attribute = rfc5652.Attribute


# Imports from RFC 5280

CertificateList = rfc5280.CertificateList


# Imports from RFC 3161

TimeStampToken = rfc3161.TimeStampToken


# Imports from RFC 4998

EvidenceRecord = rfc4998.EvidenceRecord


# TimeStampedData

class Attributes(univ.SetOf):
    componentType = Attribute()
    subtypeSpec = constraint.ValueSizeConstraint(1, MAX)


class TimeStampAndCRL(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('timeStamp', TimeStampToken()),
        namedtype.OptionalNamedType('crl', CertificateList())
    )


class TimeStampTokenEvidence(univ.SequenceOf):
    componentType = TimeStampAndCRL()
    subtypeSpec = constraint.ValueSizeConstraint(1, MAX)


class OtherEvidence(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('oeType', univ.ObjectIdentifier()),
        namedtype.NamedType('oeValue', univ.Any(),
            openType=opentype.OpenType('oeType', otherEvidenceMap))
    )


class Evidence(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('tstEvidence',
            TimeStampTokenEvidence().subtype(implicitTag=tag.Tag(
                tag.tagClassContext, tag.tagFormatSimple, 0))),
        namedtype.NamedType('ersEvidence',
            EvidenceRecord().subtype(implicitTag=tag.Tag(
                tag.tagClassContext, tag.tagFormatSimple, 1))),
        namedtype.NamedType('otherEvidence',
            OtherEvidence().subtype(implicitTag=tag.Tag(
                tag.tagClassContext, tag.tagFormatConstructed, 2)))
    )


class MetaData(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('hashProtected', univ.Boolean()),
        namedtype.OptionalNamedType('fileName', char.UTF8String()),
        namedtype.OptionalNamedType('mediaType', char.IA5String()),
        namedtype.OptionalNamedType('otherMetaData', Attributes())
    )


class TimeStampedData(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('version',
            univ.Integer(namedValues=namedval.NamedValues(('v1', 1)))),
        namedtype.OptionalNamedType('dataUri', char.IA5String()),
        namedtype.OptionalNamedType('metaData', MetaData()),
        namedtype.OptionalNamedType('content', univ.OctetString()),
        namedtype.NamedType('temporalEvidence', Evidence())
    )


id_ct_timestampedData = univ.ObjectIdentifier('1.2.840.113549.1.9.16.1.31')


# Update the CMS Content Types Map

_cmsContentTypesMapUpdate = {
    id_ct_timestampedData: TimeStampedData(),
}

cmsContentTypesMap.update(_cmsContentTypesMapUpdate)
